%% Nome Completo:
%% RA:

%% Este eh um modelo para implementacao dos jogadores. Nele voce
%% encontra a assinatura das 3 definicoes requeridas pelo gerenciador
%% de campeonato (start/5, play/1, last_play/1). Observe que este
%% arquivo contem apenas definicoes validas, mas nao implementa um
%% jogador que realize jogadas validas. 
%%
%% IMPORTANTE: 
%%
%% 1 - As chamadas feitas ao jogador devem ser respondidas em, no
%% maximo, 1 segundo. Extrapolar este tempo resultara na
%% desclassificacao do jogador.
%%
%% 2 - Nao use acentuacao, mesmo dentro dos comentarios.
%%
%% 3 - Seu jogador nao deve imprimir nada na tela. O predicado
%% print_field (e seus auxiliares) devem usados apenas para "debug"
%% (via linha de comando no swi-prolog).
%%
%% RECOMENDACOES:
%%
%% 1 - Represente a bola por um fato ball/1. Exemplo:
%%
%%   ball([4,5]).
%%
%% 2 - Represente os robos vermelhos e azuis de umas das duas
%% seguintes formas:
%%
%%   2.1 - Atraves de fatos isolados, um para cada robo. Neste caso use
%%   os nomes blue/1 e red/1. Exemplos:
%%
%%     blue([1,2]).
%%     blue([2,8]).
%%     ...
%%     blue([3,4]).
%%     
%%     red([7,3]).
%%     red([8,5]).
%%     ...
%%     red([8,5]).
%%
%%   2.2 - Atraves de listas, uma para cada cor de robo. Neste caso
%%   use os nomes reds/1 e blues/1. Exemplos:
%%
%%     blues([[2,1],[3,4],[1,3],[1,5],[1,7],[1,8],[2,2],...,[2,3]]).
%%     reds([[5,3],[5,2],[5,6],[5,7],[5,8],[6,1],[6,2],...,[6,4]]).
%%
%%   Observacao: em ambas as representacoes, as posicoes do campo de
%%   batalha devem ser representadas na forma [linha,coluna].
%%
%% 3 - Represente o numero de linhas e colunas atraves dos fatos
%% rows/1 e columns/1. Exemplo:
%%
%%   rows(8).
%%   columns(7).

%% Este predicado eh chamado apenas uma vez por partida pelo
%% gerenciador de campeonatos. Eh a primeira chamada feita ao jogador
%% no inicio de cada partida.  Eh atraves desta chamada que o jogador
%% toma conhecimento sobre o campo de batalha, que iniciara a
%% partida. O primeiro e o segundo parametros referem-se,
%% respectivamente, aos numeros de linhas e de colunas do campo de
%% batalha. O terceiro parametro refere-se a posicao inicial da bola,
%% representada por uma lista com dois inteiros no formato
%% [linha,coluna]. Os dois ultimos parametros sao, respectivamente, as
%% listas com as posicoes iniciais dos robos azuis (robos do jogador)
%% e dos robos vermelhos (robos do adversario), onde cada posicao eh
%% representada no formato [linha,coluna]. O predicado start/5 deve
%% ser sempre satisfeito, retornando yes.
start(Rows, Columns, Ball, Blues, Reds).

%% Esta definicao eh utilizada pelo gerenciador para indicar a ultima
%% jogada realizada pelo jogador oponente (robos vermelhos), no
%% formato [Old, New], onde Old e New sao posicoes do campo de batalha
%% e cada posicao eh representada no formato [linha,coluna]. O
%% predicado last_play/1 deve ser sempre satisfeito, retornando yes.
last_play([Old,New]).

%% Esta definicao eh utilizada pelo gerenciador para solicitar a
%% proxima jogada (dos robos azuis). O jogador deve retornar uma lista
%% contendo duas posicoes (Old e New), onde cada posicao eh
%% representada no formato [linha,coluna].
play([Old,New]).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%% Interface com o usuario %%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Predicados auxiliares (interger e integerZ).

% Fornece um numero inteiro entre um certo valor minimo e um valor
% maximo (inclusive ambos os extremos)

integer(Min,_,Min).
integer(Min,Max,_) :- Min >= Max, !, fail.
integer(Min,Max,X) :- Min1 is Min+1, integer(Min1,Max,X).

% Fornece um numero inteiro entre um certo valor minimo e um valor
% maximo (inclusive ambos os extremos), em ordem decrescente

integerZ(Max,_,Max).
integerZ(Max,Min,_) :- Max =< Min, !, fail.
integerZ(Max,Min,X) :- Max1 is Max-1, integerZ(Max1,Min,X).

% Imprime o campo de batalha. Ha duas formas de uso:
%
% Mode = 1: robos representados como fatos individuais (red/1 e
% blue/1). Exemplo de uso:
%   ?- print_field(1).
%
% Mode = 2: robos representados como listas, uma para cada cor (reds/1
% e blues/1). Exemplo de uso: 
%   ?- print_field(2).

print_field(Mode) :- rows(R), R1 is R + 1, columns(C), integerZ(R1,0,X), nl,
	headline(X), integer(1,C,Y), print_position(Mode,X,Y), fail.
print_field(_).

% Imprime a numeracao das linhas

headline(X) :- X < 10, write(0), fail.
headline(X) :- write(X), tab(3).

% Imprime os robos no campo de batalha, quando os robos sao
% representados individualmente por fatos (um fato para cada um dos
% robos) [Mode = 1].

print_position(1,X,Y) :- red([X,Y]), ball([X,Y]), write('R '), !.
print_position(1,X,Y) :- blue([X,Y]), ball([X,Y]), write('B '), !.
print_position(1,X,Y) :- red([X,Y]), write('r '), !.
print_position(1,X,Y) :- blue([X,Y]), write('b '), !.

% Imprime os robos no campo de batalha, quando os robos sao
% representados coletavamente por listas (uma para cada cor de robos)
% [Mode = 2].

print_position(2,X,Y) :- reds(R), member([X,Y],R), ball([X,Y]),
	write('R '), !.
print_position(2,X,Y) :- blues(B), member([X,Y],B), ball([X,Y]),
	write('B '), !.
print_position(2,X,Y) :- reds(R), member([X,Y],R), write('r '), !.
print_position(2,X,Y) :- blues(B), member([X,Y],B), write('b '), !.

% Imprime a bola e as posicoes vazias

print_position(_,X,Y) :- ball([X,Y]), write('o '), !.
print_position(_,_,_) :- put_char(183), write(' ').
